//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/CatSample/ItemWithParticlesCatalog.h
//! @brief     Defines class ItemWithParticlesCatalog
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_CATSAMPLE_ITEMWITHPARTICLESCATALOG_H
#define BORNAGAIN_GUI_MODEL_CATSAMPLE_ITEMWITHPARTICLESCATALOG_H

#include <QString>
#include <QVector>

class ItemWithParticles;
class MaterialItems;

class ItemWithParticlesCatalog {
public:
    using CatalogedType = ItemWithParticles;

    // Do not change the numbering! It is serialized!
    enum class Type : uint8_t { Particle = 1, Composition = 2, CoreShell = 3, Mesocrystal = 4 };

    struct UiInfo {
        QString menuEntry;
        QString description;
        QString iconPath;
    };

    //! Creates the item of the given type.
    //!
    //! If type is "None", a nullptr is returned.
    static ItemWithParticles* create(Type type, const MaterialItems* materials);

    //! Available types of items, sorted as expected in the UI (e.g. in combo box)
    static QVector<Type> types();

    //! Available types of assembly items, sorted as expected in the UI (e.g. in combo box)
    static QVector<Type> assemblyTypes();

    //! UiInfo on the given type.
    static UiInfo uiInfo(Type t);

    //! Returns the enum type of the given item.
    static Type type(const ItemWithParticles* item);
};

#endif // BORNAGAIN_GUI_MODEL_CATSAMPLE_ITEMWITHPARTICLESCATALOG_H
