//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/IFormFactorPolyhedron.h
//! @brief     Defines interface IFormFactorPolyhedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H

#include "Sample/Particle/IFormFactor.h"
#include <ff/Topology.h> // needed by all child classes
#include <memory>

namespace ff {

class Polyhedron;

}

//! A polyhedron, for form factor computation.

class IFormFactorPolyhedron : public IFormFactor {
public:
    IFormFactorPolyhedron(const std::vector<double>& PValues);
    ~IFormFactorPolyhedron() override;

    double bottomZ(const IRotation* rotation) const override;
    double topZ(const IRotation* rotation) const override;

    complex_t formfactor_at_bottom(C3 q) const override;

    double volume() const override;
    double radialExtension() const override;

protected:
    void setPolyhedron(const ff::Topology& topology, double z_bottom,
                       const std::vector<R3>& vertices) const;
    void shift(double z_center) const;

    // private:
    mutable std::unique_ptr<ff::Polyhedron> pimpl;
    mutable std::vector<R3> m_vertices; //! for topZ, bottomZ computation only
    mutable double m_z_bottom;
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPOLYHEDRON_H
#endif // USER_API
