//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/PlatonicOctahedron.h
//! @brief     Defines class PlatonicOctahedron
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_PLATONICOCTAHEDRON_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_PLATONICOCTAHEDRON_H

#include "Sample/HardParticle/IFormFactorPolyhedron.h"

//! A truncated bifrustum with quadratic base.
//! @ingroup hardParticle

class PlatonicOctahedron : public IFormFactorPolyhedron {
public:
    PlatonicOctahedron(double edge);
#ifndef USER_API
    PlatonicOctahedron(std::vector<double> P);

    PlatonicOctahedron* clone() const override
    {
        return new PlatonicOctahedron(m_edge);
    }
    std::string className() const final
    {
        return "PlatonicOctahedron";
    }
    std::vector<ParaMeta> parDefs() const final
    {
        return {{"Edge", "nm"}};
    }

    double edge() const
    {
        return m_edge;
    }
    double height() const
    {
        return sqrt(1 / 2.) * m_edge;
    }

    std::string validate() const override;

private:
    static const ff::Topology topology;

    const double& m_edge;
#endif // USER_API
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_PLATONICOCTAHEDRON_H
