//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/Prism6.cpp
//! @brief     Implements class Prism6.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/Prism6.h"

Prism6::Prism6(const std::vector<double> P)
    : IFormFactorPrism(P)
    , m_base_edge(m_P[0])
    , m_height(m_P[1])
{
    validateOrThrow();
}

Prism6::Prism6(double base_edge, double height)
    : Prism6(std::vector<double>{base_edge, height})
{
}

std::string Prism6::validate() const
{
    std::vector<std::string> errs;
    requestGt0(errs, m_base_edge, "base_edge");
    requestGt0(errs, m_height, "height");
    if (!errs.empty())
        return jointError(errs);

    double a = m_base_edge;
    double as = a * sqrt(3) / 2;
    double ac = a / 2;
    std::vector<R3> V{{a, 0., 0.},  {ac, as, 0.},   {-ac, as, 0.},
                      {-a, 0., 0.}, {-ac, -as, 0.}, {ac, -as, 0.}};
    setPrism(false, V);

    m_validated = true;
    return "";
}
