//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/TruncatedCube.h
//! @brief     Defines class TruncatedCube.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_TRUNCATEDCUBE_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_TRUNCATEDCUBE_H

#include "Sample/HardParticle/IFormFactorPolyhedron.h"

//! A cube, with tetrahedral truncation of all corners
//! @ingroup hardParticle

class TruncatedCube : public IFormFactorPolyhedron {
public:
    TruncatedCube(double length, double removed_length);
#ifndef USER_API
    TruncatedCube(std::vector<double> P);

    TruncatedCube* clone() const override
    {
        return new TruncatedCube(m_length, m_removed_length);
    }
    std::string className() const final
    {
        return "TruncatedCube";
    }
    std::vector<ParaMeta> parDefs() const final
    {
        return {{"Length", "nm"}, {"RemovedLength", "nm"}};
    }

    double length() const
    {
        return m_length;
    }
    double removedLength() const
    {
        return m_removed_length;
    }
    std::string validate() const override;

private:
    static const ff::Topology topology;
    const double& m_length;
    const double& m_removed_length;
#endif // USER_API
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_TRUNCATEDCUBE_H
