#ifndef BORNAGAIN_TESTS_UNIT_NUMERIC_MULTIQTEST_H
#define BORNAGAIN_TESTS_UNIT_NUMERIC_MULTIQTEST_H

#include "Tests/GTestWrapper/google_test.h"
#include <functional>
#include <heinz/Complex.h>
#include <heinz/Vectors3D.h>

//! Driver for FormFactorSpecializationTest and FormFactorSymmetryTest.

namespace formfactorTest {

//! Runs callback function "run_one_test(q)" for a huge number of different vectors q.
void run_test_for_many_q(std::function<void(C3)> run_one_test, double qmag_min, double qmag_max,
                         bool real_only = false);

} // namespace formfactorTest


class MultiQTest : public testing::Test {
protected:
    //! Tests whether form factors are almost equal for various real q
    void test_complexEq_at_realQ(std::function<complex_t(const C3&)> f0,
                                 std::function<complex_t(const C3&)> f1, double eps, double qmag1,
                                 double qmag2)
    {
        formfactorTest::run_test_for_many_q([&](C3 q) { complexEq(q, f0(q), f1(q), eps); }, qmag1,
                                            qmag2, true);
    }
    //! Tests whether absolute values of form factors are almost equal for various real q
    void test_absEq_at_realQ(std::function<complex_t(const C3&)> f0,
                             std::function<complex_t(const C3&)> f1, double eps, double qmag1,
                             double qmag2)
    {
        formfactorTest::run_test_for_many_q([&](C3 q) { absEq(q, f0(q), f1(q), eps); }, qmag1,
                                            qmag2, true);
    }

private:
    //! Tests whether c0 and c1 are almost equal.
    //! Parameter q is transmitted for use in error message only.
    void complexEq(C3 q, complex_t c0, complex_t c1, double eps)
    {
        const double avge = (std::abs(c0) + std::abs(c1)) / 2;
        const double precision = std::max(1e-16, eps * avge);
        EXPECT_NEAR(real(c0), real(c1), precision) << "q=" << q << "\n";
        EXPECT_NEAR(imag(c0), imag(c1), precision) << "q=" << q << "\n";
    }
    void absEq(C3 q, complex_t c0, complex_t c1, double eps)
    {
        const double avge = (std::abs(c0) + std::abs(c1)) / 2;
        const double precision = std::max(1e-16, eps * avge);
        EXPECT_NEAR(abs(c0), abs(c1), precision) << "q=" << q << "\n";
    }
};

#endif // BORNAGAIN_TESTS_UNIT_NUMERIC_MULTIQTEST_H
